import io
import os
import unittest

import cv2
import numpy as np

from app import app
from processing.ainnotate import ainnotate
from utils.constants import UPLOAD_FOLDER

# test_app.py


class FlaskAppTests(unittest.TestCase):

    def setUp(self):
        app.config['TESTING'] = True
        app.config['UPLOAD_FOLDER'] = 'test_uploads'
        self.client = app.test_client()
        os.makedirs(app.config['UPLOAD_FOLDER'], exist_ok=True)

    def tearDown(self):
        for file in os.listdir(app.config['UPLOAD_FOLDER']):
            file_path = os.path.join(app.config['UPLOAD_FOLDER'], file)
            os.unlink(file_path)
        os.rmdir(app.config['UPLOAD_FOLDER'])

    def test_home(self):
        response = self.client.get('/')
        self.assertEqual(response.status_code, 200)
        self.assertIn(b'<!DOCTYPE html>', response.data)

    def test_running(self):
        response = self.client.get('/test')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.data, b'Server is running')

    def test_upload_no_file_part(self):
        response = self.client.post('/upload', data={})
        self.assertEqual(response.status_code, 400)
        self.assertIn(b'Redirect', response.data)

    def test_upload_no_selected_file(self):
        data = {'image': (io.BytesIO(b''), '')}
        response = self.client.post(
            '/upload', data=data, content_type='multipart/form-data')
        self.assertEqual(response.status_code, 400)
        self.assertIn(b'Redirect', response.data)

    def test_upload_successful(self):
        data = {'image': (io.BytesIO(b'test image content'), 'test_image.png')}
        response = self.client.post(
            '/upload', data=data, content_type='multipart/form-data')
        self.assertEqual(response.status_code, 302)
        self.assertIn(b'/segment', response.data)

    def test_segment(self):
        response = self.client.get('/segment')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.data.decode('utf-8'), '{}\n')

class TestAinnotate(unittest.TestCase):

    def setUp(self):
        os.makedirs(UPLOAD_FOLDER, exist_ok=True)
        self.test_image_path = os.path.join(UPLOAD_FOLDER, 'test_image.png')
        # Create a dummy image for testing
        dummy_image = np.ones((100, 100, 3), dtype=np.uint8) * 255
        cv2.imwrite(self.test_image_path, dummy_image)

    def tearDown(self):
        if os.path.exists(self.test_image_path):
            os.remove(self.test_image_path)
        if os.path.exists(UPLOAD_FOLDER):
            os.rmdir(UPLOAD_FOLDER)

    def test_ainnotate_valid_image(self):
        file, result = ainnotate('test_image.png')
        self.assertEqual(file, 'test_image.png')
        self.assertIsNotNone(result)
        self.assertIn('rois', result)
        self.assertIn('masks', result)
        self.assertIn('class_ids', result)
        self.assertIn('slice_scores', result)
        self.assertIn('infarct_scores', result)
        self.assertIn('risk_scores', result)

    def test_ainnotate_non_existent_file(self):
        file, result = ainnotate('non_existent.png')
        self.assertEqual(file, 'non_existent.png')
        self.assertIsNone(result)

    def test_ainnotate_invalid_image(self):
        invalid_image_path = os.path.join(UPLOAD_FOLDER, 'invalid_image.txt')
        with open(invalid_image_path, 'w') as f:
            f.write('This is not an image file.')
        file, result = ainnotate('invalid_image.txt')
        self.assertEqual(file, 'invalid_image.txt')
        self.assertIsNone(result)
        os.remove(invalid_image_path)


if __name__ == '__main__':
    unittest.main()
