import torch
import torch.nn as nn
import torch.nn.functional as F


# Building blocks of U-Net
class DoubleConv2(nn.Module):
    def __init__(self, in_channels, out_channels):
        super().__init__()
        self.double_conv = nn.Sequential(
            nn.Conv2d(in_channels, out_channels, kernel_size=3, padding=1),
            nn.BatchNorm2d(out_channels),
            nn.LeakyReLU(inplace=True),
            nn.Conv2d(out_channels, out_channels, kernel_size=3, padding=1),
            nn.BatchNorm2d(out_channels),
            nn.LeakyReLU(inplace=True)
        )

    def forward(self, x):
        return self.double_conv(x)


# Blocks for down-sampling
class Down2(nn.Module):
    def __init__(self, in_channels, out_channels, down_with_conv2d=False):
        super().__init__()
        self.maxpool_conv = nn.Sequential(
            DoubleConv2(in_channels, out_channels),
            nn.MaxPool2d(2)
        )

    def forward(self, x):
        return self.maxpool_conv(x)

# Blocks for up-sampling


class Up2(nn.Module):
    def __init__(self, in_channels, out_channels, bilinear=False):
        super().__init__()

        if bilinear:
            self.up = nn.Upsample(
                scale_factor=2, mode='bilinear', align_corners=True)
        else:
            self.up = nn.ConvTranspose2d(
                in_channels, in_channels // 2, kernel_size=2, stride=2)

        self.conv = DoubleConv2(in_channels, out_channels)

    def forward(self, x1, x2):
        x1 = self.up(x1)
        diffY = torch.tensor([x2.size()[2] - x1.size()[2]])
        diffX = torch.tensor([x2.size()[3] - x1.size()[3]])

        x1 = F.pad(x1, [diffX // 2, diffX - diffX // 2,
                        diffY // 2, diffY - diffY // 2])
        x = torch.cat([x2, x1], dim=1)
        return self.conv(x)

# Output layer


class OutConv2(nn.Module):
    def __init__(self, in_channels, out_channels):
        super(OutConv2, self).__init__()
        self.conv = nn.Conv2d(in_channels, out_channels, kernel_size=1)

    def forward(self, x):
        return self.conv(x)


# Channel numbers
ChN = [64, 128, 256, 512, 1024]


# Creating U-Net
class UNet2(nn.Module):
    def __init__(self, n_channels, n_classes, bilinear=False):
        super(UNet2, self).__init__()
        self.n_channels = n_channels
        self.n_classes = n_classes
        self.inc = DoubleConv2(n_channels, ChN[0])
        self.down1 = Down2(ChN[0], ChN[1])
        self.down2 = Down2(ChN[1], ChN[2])
        self.down3 = Down2(ChN[2], ChN[3])
        self.down4 = Down2(ChN[3], ChN[4])
        self.up1 = Up2(ChN[4], ChN[3], bilinear)
        self.up2 = Up2(ChN[3], ChN[2], bilinear)
        self.up3 = Up2(ChN[2], ChN[1], bilinear)
        self.up4 = Up2(ChN[1], ChN[0], bilinear)
        self.outc = OutConv2(ChN[0], n_classes)

    def forward(self, x):
        x1 = self.inc(x)
        x2 = self.down1(x1)
        x3 = self.down2(x2)
        x4 = self.down3(x3)
        x5 = self.down4(x4)
        x = self.up1(x5, x4)
        x = self.up2(x, x3)
        x = self.up3(x, x2)
        x = self.up4(x, x1)
        logits = self.outc(x)
        return logits
