import os

import cv2
import numpy as np
from rembg import remove
from torch import float32, from_numpy, no_grad, sigmoid

from models.load_models import device, net1, net2, net3
from utils.constants import UPLOAD_FOLDER
from utils.helpers import (calculate_score, cv2_RGBA2RGB, delete_buff_image,
                           reshape_and_threshold)


# Method to ainnotate the image
def ainnotate(file, thresh=50.0):
    try:
        print(f"Processing file: {file}")
        img = cv2.imread(os.path.join(UPLOAD_FOLDER, file))
        if img is None:
            print(f"Error while reading file: {file}")
            return file, None

        img = remove(img)
        filtered = np.ones((img.shape[0], img.shape[1]), dtype=np.uint8) * 255
        filtered[img[:, :, 0] < 10] = 0
        num_labels, labels_im = cv2.connectedComponents(filtered)

        result = {
            'rois': [],
            'masks': [],
            'class_ids': [],
            'slice_scores': [],
            'infarct_scores': [],
            'risk_scores': [],
        }
        cut_names = []
        counter = 0

        for n in range(1, num_labels + 1):
            ThisLabel = (labels_im == n)

            if ThisLabel.sum() > 0.005 * (labels_im.shape[0] * labels_im.shape[1]):
                a = np.where(ThisLabel != 0)
                Left = np.min(a[0])
                Right = np.max(a[0])
                Down = np.min(a[1])
                Up = np.max(a[1])
                if (((Up - Down) > 0.01 * labels_im.shape[1]) and
                        ((Right - Left) > 0.01 * labels_im.shape[0]) and
                        ((Right - Left) < 5 * (Up - Down)) and ((Up - Down) < 5 * (Right - Left))):
                    counter += 1
                    CutImg = img[Left:Right, Down:Up, :]
                    file_name = 'cut_' + str(n) + '.png'
                    cut_names.append(file_name)

                    WidthScale = CutImg.shape[0] / 224.0
                    HeightScale = CutImg.shape[1] / 224.0
                    CutImg = cv2_RGBA2RGB(CutImg)
                    CutImg = cv2.resize(CutImg, (224, 224))
                    Data = np.reshape(CutImg, [1, 224, 224, 3])
                    Data = np.swapaxes(Data, 1, 3)

                    Data = from_numpy(Data)
                    Data = Data.to(device=device, dtype=float32) / 127.5 - 1.0

                    with no_grad():
                        out1 = net1(Data)
                        prob1 = sigmoid(out1).cpu().detach().numpy()
                        out2 = net2(Data)
                        prob2 = sigmoid(out2).cpu().detach().numpy()
                        out3 = net3(Data)
                        prob3 = sigmoid(out3).cpu().detach().numpy()
                        prob_comb = sigmoid(out1.add(out2).add(
                            out3) / 3).cpu().detach().numpy()

                    mask_pred = out1.cpu().detach().add(out2.cpu().detach()).add(
                        out3.cpu().detach()).numpy() / 3.0
                    prob_comb = np.reshape(np.swapaxes(
                        prob_comb, 1, 2), [224, 224, 3])
                    prob1 = np.reshape(np.swapaxes(prob1, 1, 2), [224, 224, 3])
                    prob2 = np.reshape(np.swapaxes(prob2, 1, 2), [224, 224, 3])
                    prob3 = np.reshape(np.swapaxes(prob3, 1, 2), [224, 224, 3])
                    ActualOut = reshape_and_threshold(mask_pred)

                    result['slice_scores'].append(
                        calculate_score(prob1[:, :, 0], prob2[:, :, 0], prob3[:, :, 0], prob_comb[:, :, 0]))
                    result['risk_scores'].append(
                        calculate_score(prob1[:, :, 2], prob2[:, :, 2], prob3[:, :, 2], prob_comb[:, :, 2]))
                    result['infarct_scores'].append(
                        calculate_score(prob1[:, :, 1], prob2[:, :, 1], prob3[:, :, 1], prob_comb[:, :, 1]))

                    for channel in range(3):
                        padded = np.pad(ActualOut[0, :, :, channel], pad_width=2, mode='constant',
                                        constant_values=0)
                        contours, _ = cv2.findContours(padded.astype(np.uint8), cv2.RETR_TREE,
                                                       cv2.CHAIN_APPROX_SIMPLE)
                        for cont in contours:
                            Points = []
                            PrevPoint = []
                            for vert in cont:
                                vert = vert[0]
                                if len(PrevPoint) == 2:
                                    if (vert[0] - PrevPoint[0]) ** 2 + (vert[1] - PrevPoint[1]) ** 2 > thresh:
                                        Points.append(
                                            int((vert[0] - 2) * HeightScale + Down))
                                        Points.append(
                                            int((vert[1] - 2) * WidthScale + Left))
                                        PrevPoint = [vert[0], vert[1]]
                                else:
                                    Points.append(
                                        int(((vert[0] - 2) * HeightScale + Down)))
                                    Points.append(
                                        int(((vert[1] - 2) * WidthScale + Left)))
                                    PrevPoint = [vert[0], vert[1]]

                            result['class_ids'].append(channel)
                            result['masks'].append(Points)
        delete_buff_image(file)
        return file, result

    except Exception as e:
        print(e)
        print("Error while processing file : ", file)
        delete_buff_image(file)
        return file, None
