import unittest

import numpy as np
import segmentation_models_pytorch as smp
import torch
import torch.nn as nn
from torch.utils.data import DataLoader

from models.unet_impl import UNet2
from train.train import train_for_epochs
from train.utils import MyDataset, MyDataset_test, device
from utils.constants import num_classes


class TestModelGeneration(unittest.TestCase):
    def setUp(self):
        # Generate random data for training and testing
        self.train_imgs = np.random.rand(100, 128, 128, 3).astype(np.float32)
        self.train_masks = np.random.randint(
            0, 2, (100, 128, 128, num_classes)).astype(np.float32)  # Binary masks
        self.test_imgs = np.random.rand(20, 128, 128, 3).astype(np.float32)
        self.test_masks = np.random.randint(
            0, 2, (20, 128, 128, num_classes)).astype(np.float32)  # Binary masks

        # Create datasets and dataloaders
        self.train_set = MyDataset(self.train_imgs, self.train_masks)
        self.test_set = MyDataset_test(self.test_imgs, self.test_masks)
        self.train_loader = DataLoader(
            dataset=self.train_set, batch_size=10, shuffle=True)
        self.test_loader = DataLoader(
            dataset=self.test_set, batch_size=4, shuffle=False)

        # Initialize models
        self.net1 = UNet2(3, num_classes).to(device)
        self.net2 = smp.UnetPlusPlus(
            encoder_name='resnet34', encoder_weights=None, classes=num_classes).to(device)
        self.net3 = smp.DeepLabV3Plus(
            encoder_name='resnet34', encoder_weights=None, classes=num_classes).to(device)

    def test_generate_and_save_models(self):
        # Define loss function
        loss_fn = nn.BCEWithLogitsLoss()

        # Train and save Net1
        print("Training Net1")
        self.net1 = train_for_epochs(
            self.net1, self.train_loader, self.test_loader, loss_fn, "models/random_net1.pt", 1, 0.0)
        torch.save(self.net1.state_dict(), "models/random_net1.pt")
        print("Net1 saved as 'models/random_net1.pt'")

        # Train and save Net2
        print("Training Net2")
        self.net2 = train_for_epochs(
            self.net2, self.train_loader, self.test_loader, loss_fn, "models/random_net2.pt", 1, 0.0)
        torch.save(self.net2.state_dict(), "models/random_net2.pt")
        print("Net2 saved as 'models/random_net2.pt'")

        # Train and save Net3
        print("Training Net3")
        self.net3 = train_for_epochs(
            self.net3, self.train_loader, self.test_loader, loss_fn, "models/random_net3.pt", 1, 0.0)
        torch.save(self.net3.state_dict(), "models/random_net3.pt")
        print("Net3 saved as 'models/random_net3.pt'")

        # Assertions to check if files are created
        self.assertTrue(torch.load("models/random_net1.pt"))
        self.assertTrue(torch.load("models/random_net2.pt"))
        self.assertTrue(torch.load("models/random_net3.pt"))


if __name__ == "__main__":
    unittest.main()
